#include "e_mod_main.h"


static int       _ngi_taskbar_border_check          (Ngi_Box *box, E_Border *bd);
static Ngi_Item *_ngi_taskbar_border_find           (Ngi_Box *box, E_Border *bd);

static int       _ngi_taskbar_border_cb_add         (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_remove      (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_iconify     (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_uniconify   (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_icon_change (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_property    (void *data, int type, void *event);
static int       _ngi_taskbar_border_cb_zone_set    (void *data, int type, void *event);
static int       _ngi_taskbar_cb_desk_show          (void *data, int type, void *event);

static void      _ngi_taskbar_item_new              (Ngi_Box *box, E_Border *bd);
static void      _ngi_taskbar_item_set_icon         (Ngi_Item *it);
static void      _ngi_taskbar_item_set_label        (Ngi_Item *it);

static void      _ngi_taskbar_item_cb_free          (Ngi_Item *it);
static void      _ngi_taskbar_item_cb_mouse_down    (Ngi_Item *it, Ecore_X_Event_Mouse_Button_Down *ev);
static void      _ngi_taskbar_item_cb_mouse_up      (Ngi_Item *it, Ecore_X_Event_Mouse_Button_Up *ev);
static void      _ngi_taskbar_item_cb_drag_start    (Ngi_Item *it);

static void      _ngi_taskbar_item_cb_drag_finished (E_Drag *drag, int dropped);

static int       _ngi_taskbar_cb_show_window        (void *data);
static void      _ngi_taskbar_cb_drop_enter         (void *data, const char *type, void *event_info);
static void      _ngi_taskbar_cb_drop_move          (void *data, const char *type, void *event_info);
static void      _ngi_taskbar_cb_drop_end           (void *data, const char *type, void *event_info);
static void      _ngi_taskbar_cb_drop_leave         (void *data, const char *type, void *event_info);


void
ngi_taskbar_init(void)
{
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_ADD, _ngi_taskbar_border_cb_add, NULL));
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_REMOVE, _ngi_taskbar_border_cb_remove, NULL));
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_ICONIFY, _ngi_taskbar_border_cb_iconify, NULL));
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_UNICONIFY, _ngi_taskbar_border_cb_uniconify, NULL));
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_ICON_CHANGE, _ngi_taskbar_border_cb_icon_change, NULL));
  ngi_config->handlers = evas_list_append
    (ngi_config->handlers, ecore_event_handler_add
     (E_EVENT_BORDER_PROPERTY, _ngi_taskbar_border_cb_property, NULL));
  ngi_config->handlers = evas_list_append
      (ngi_config->handlers, ecore_event_handler_add
      (E_EVENT_BORDER_ZONE_SET, _ngi_taskbar_border_cb_zone_set, NULL));
  ngi_config->handlers = evas_list_append
      (ngi_config->handlers, ecore_event_handler_add
      (E_EVENT_DESK_SHOW, _ngi_taskbar_cb_desk_show, NULL));
}

void
ngi_taskbar_new(Ng *ng, Config_Box *cfg)
{
  Ngi_Box *box = ngi_box_new(ng);
  if(!box) return;
  
  box->cfg = cfg;
  cfg->box = box;
  
  // "enlightenment/border", "enlightenment/desktop",
  const char *drop[] = {"text/uri-list","text/x-moz-url", "enlightenment/x-file"};
  box->drop_handler = e_drop_handler_add(E_OBJECT(ng->win->fake_iwin), box,
                                             _ngi_taskbar_cb_drop_enter, _ngi_taskbar_cb_drop_move,
                                             _ngi_taskbar_cb_drop_leave, _ngi_taskbar_cb_drop_end,
                                             drop, 3, 0, 0, 0, 0);

  
  ngi_taskbar_fill(box);
}

void
ngi_taskbar_fill(Ngi_Box *box)
{
  E_Border *bd;
  E_Border_List *bl;

  bl = e_container_border_list_first(box->ng->zone->container);

  while((bd = e_container_border_list_next(bl)))
    {
      if(box->ng->zone == bd->zone)
	_ngi_taskbar_item_new(box, bd);
    }

  e_container_border_list_free(bl);
}

void
ngi_taskbar_empty(Ngi_Box *box)
{
  while(box->items)
    {
      ngi_item_remove((Ngi_Item*) box->items->data, 1);
    }
}

void
ngi_taskbar_remove(Ngi_Box *box)
{
  e_drop_handler_del(box->drop_handler);

  ngi_taskbar_empty(box);
  
  ngi_box_free(box);
}

static int
_ngi_taskbar_border_check(Ngi_Box *box, E_Border *bd)
{
  if(box->cfg->taskbar_show_desktop && bd->desk != e_desk_current_get(box->ng->zone)) return 0;
  
  if(box->cfg->taskbar_show_iconified == 0 && bd->iconic) return 0;
  
  if(box->cfg->taskbar_show_iconified == 2 && !bd->iconic) return 0;
  
  if((box->cfg->taskbar_skip_dialogs && bd->client.netwm.type == ECORE_X_WINDOW_TYPE_DIALOG)
     || (bd->user_skip_winlist)
     || (bd->client.netwm.state.skip_taskbar))
    return 0;

  if(bd->client.icccm.transient_for && bd->client.icccm.class && !strcmp(bd->client.icccm.class, "Xitk")) return 0;
  
  return 1;
}

static Ngi_Item *
_ngi_taskbar_border_find(Ngi_Box *box, E_Border *bd)
{
  Evas_List *l;

  for (l = box->items; l; l = l->next)
    if(((Ngi_Item*)(l->data))->border == bd)
      return (Ngi_Item*) l->data;

  return NULL;
}


/* *************************  DND  CALLBACKS  ************************** */

static int
_ngi_taskbar_pos_set(Ngi_Box *box, int pos)
{
  box->ng->pos = pos + box->start;

  return 1;
}

static void
_ngi_taskbar_cb_drop_enter(void *data, const char *type, void *event_info)
{
  E_Event_Dnd_Enter *ev = (E_Event_Dnd_Enter*) event_info;
  Ngi_Box *box = (Ngi_Box*) data;
  Ng *ng = box->ng;

  _ngi_taskbar_pos_set(box, ng->horizontal ? ev->x : ev->y);

  ng->item_active = ngi_box_item_at_position_get(box);

  if(!ng->item_active) return; /* FIXME set edge_in before ?*/

  ITEM_MOUSE_IN(ng->item_active);

  if(box->dnd_timer) ecore_timer_del(box->dnd_timer);
  box->dnd_timer = ecore_timer_add(0.6, _ngi_taskbar_cb_show_window, box);
  //ng->mouse_in = 0;
  //ng->edge_in = 0;
  ngi_mouse_in(ng);
}

static void
_ngi_taskbar_cb_drop_leave(void *data, const char *type, void *event_info)
{
  Ngi_Box *box = (Ngi_Box*) data;
  Ng *ng = box->ng;

  if(box->dnd_timer) ecore_timer_del(box->dnd_timer);
  box->dnd_timer = NULL;

  ng->item_active = NULL;

  ngi_mouse_out(ng);
}

static void
_ngi_taskbar_cb_drop_move(void *data, const char *type, void *event_info)
{
  E_Event_Dnd_Move *ev = (E_Event_Dnd_Move*) event_info;
  Ngi_Box *box = (Ngi_Box*) data;
  Ng *ng = box->ng;
  Ngi_Item *it;

  _ngi_taskbar_pos_set(box, ng->horizontal ? ev->x : ev->y);

  it = ngi_box_item_at_position_get(box);

  if(!it || !ng->item_active) return;

  if(ng->item_active != it)
    {
    if(box->dnd_timer) ecore_timer_del(box->dnd_timer);

      ITEM_MOUSE_OUT(ng->item_active);
      ITEM_MOUSE_IN(it);

      ng->item_active = it;

      box->dnd_timer = ecore_timer_add(0.6, _ngi_taskbar_cb_show_window, box);
    }

  ngi_animate(ng);
}

static void
_ngi_taskbar_cb_drop_end(void *data, const char *type, void *event_info)
{
  Ngi_Box *box = (Ngi_Box*) data;
  Ng *ng = box->ng;

  if(!ng->item_active) return;

  ng->item_active = NULL;

  if(box->dnd_timer) ecore_timer_del(box->dnd_timer);
  box->dnd_timer = NULL;

  ngi_mouse_out(ng);
}

static int
_ngi_taskbar_cb_show_window(void *data)
{
  Ngi_Box *box = (Ngi_Box*) data;
  E_Desk *desk;
  Ng *ng = box->ng;
  Ngi_Item *it = ng->item_active;

  if(!it) return 0;

  desk = e_desk_current_get(it->border->zone);

  if(desk != it->border->desk && !it->border->sticky)
    {
      e_desk_show(it->border->desk);
    }

  if(it->border->shaded)
    {
      e_border_unshade(it->border, E_DIRECTION_UP);
    }

  if(!it->border->visible)
    {
      e_border_show(it->border);
    }

  if(it->border->iconic)
    {
      e_border_uniconify(it->border);

      /*if(it->border_was_fullscreen)
        {
        e_border_fullscreen(bd, e_config->fullscreen_policy);
        it->border_was_fullscreen = 0;
        }*/
    }
  else
    {
      e_border_raise(it->border);
    }

  e_border_focus_set(it->border, 1, 1);

  /* ngi_item_signal_emit(it, "e,state,focused");*/

  box->dnd_timer = NULL;
  return 0;
}


/* ************************** BORDER CALLBACKS ************************* */

static int
_ngi_taskbar_border_cb_add(void *data, int type, void *event)
{
  E_Event_Border_Add *ev = (E_Event_Border_Add*) event;
  Ngi_Box *box;
  Ng *ng;
  Evas_List *l, *ll;
  
  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;

      if(ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          _ngi_taskbar_item_new(box, ev->border);
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_remove(void *data, int type, void *event)
{
  E_Event_Border_Remove *ev = (E_Event_Border_Remove*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      if(ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          it = _ngi_taskbar_border_find(box, ev->border);
          if (it)
            {
              if(it->is_single_instance_app)
                ngi_launcher_handle_border_event(ng, ev->border, 0);

              ngi_item_remove(it, 0);
            }
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_uniconify(void *data, int type, void *event)
{
  E_Event_Border_Iconify *ev = (E_Event_Border_Iconify*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      if(ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          it = _ngi_taskbar_border_find(box, ev->border);
          if (it) edje_object_signal_emit(it->obj, "e,state,item_uniconify", "e");
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_iconify(void *data, int type, void *event)
{
  E_Event_Border_Uniconify *ev = (E_Event_Border_Uniconify*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      if(ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          it = _ngi_taskbar_border_find(box, ev->border);
          if (it) edje_object_signal_emit(it->obj, "e,state,item_iconify", "e");
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_icon_change(void *data, int type, void *event)
{
  E_Event_Border_Icon_Change *ev = (E_Event_Border_Icon_Change*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      if(ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          it = _ngi_taskbar_border_find(box, ev->border);
          if (!it) continue;

          ngi_item_del_icon(it);
          _ngi_taskbar_item_set_icon(it);

          // ngi_item_app_is_single_instance(it, NULL); /* FIXME here? */
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_property(void *data, int type, void *event)
{
  E_Event_Border_Property *ev = (E_Event_Border_Property*) event;
  E_Border *bd = ev->border;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      if (ng->zone != ev->border->zone) continue;

      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if (box->cfg->type != taskbar) continue;

          it = _ngi_taskbar_border_find(box, bd);
          if (it && it->usable)
            {
              if (it->is_single_instance_app)
                ngi_launcher_handle_border_event(ng, bd, 0);
                
              if (!_ngi_taskbar_border_check(box, bd))
                ngi_item_remove(it, 0);
              else
                _ngi_taskbar_item_set_label(it);
            }
          else if (!it)
            {
              _ngi_taskbar_item_new(box, ev->border);
            }
        }
    }
  return 1;
}

static int
_ngi_taskbar_border_cb_zone_set(void *data, int type, void *event)
{
  E_Event_Border_Zone_Set *ev = (E_Event_Border_Zone_Set*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll;

  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar) continue;

          if(ng->zone == ev->border->zone)
            {
              e_border_zone_set(ev->border, ng->zone);
              _ngi_taskbar_item_new(box, ev->border);
            }
          else
            {
              it = _ngi_taskbar_border_find(box, ev->border);
              if(it) ngi_item_remove(it, 0);
            }
        }
    }
  return 1;
}

static int
_ngi_taskbar_cb_desk_show(void *data, int type, void *event)
{
  E_Event_Border_Zone_Set *ev = (E_Event_Border_Zone_Set*) event;
  Ng *ng;
  Ngi_Box *box;
  Ngi_Item *it;
  Evas_List *l, *ll, *lll;
  E_Desk *desk;
  int changed = 0;
  
  for (l = ngi_config->instances; l; l = l->next)
    {
      ng = (Ng*) l->data;
      for(ll = ng->boxes; ll; ll = ll->next)
        {
          box = ll->data;
          if(box->cfg->type != taskbar || !box->cfg->taskbar_show_desktop) continue;

          desk = e_desk_current_get(ng->zone);
          
          for(lll = box->items; lll; lll = lll->next)
            {
              it = lll->data;
              if(it->border->desk == desk)
                {
                  changed = 1;
                  it->scale = 1.0;
                }
              else
                {
                  changed = 1;
                  it->scale = 0.0;
                }     
            }
          if(changed) ngi_thaw(ng);
          changed = 0;
        }
    }
  return 1;
}


/* ***************************  TASKBAR  ITEM  *************************** */

static void
_ngi_taskbar_item_new(Ngi_Box *box, E_Border *bd)
{
  //printf("_ngi_taskbar_item_new\n");
  
  Ngi_Item *it, *l_it;
  Evas_List *l;
  char *val;
  Ng *ng = box->ng;

  if(!_ngi_taskbar_border_check(box, bd) ||
      _ngi_taskbar_border_find(box, bd)) return;

  it = ngi_item_new(box);
  it->type = taskbar_item;
  
  e_object_ref(E_OBJECT(bd));

  it->border = bd;

  it->cb_free       = _ngi_taskbar_item_cb_free;
  it->cb_mouse_in   = ngi_item_mouse_in;
  it->cb_mouse_out  = ngi_item_mouse_out;
  it->cb_mouse_down = _ngi_taskbar_item_cb_mouse_down;
  it->cb_mouse_up   = _ngi_taskbar_item_cb_mouse_up;
  it->cb_drag_start = _ngi_taskbar_item_cb_drag_start;

  _ngi_taskbar_item_set_icon(it);
  _ngi_taskbar_item_set_label(it);

  if(it->border->client.icccm.class)
    {
      for(l = box->items; l; l = l->next)
        {
          l_it = (Ngi_Item*) l->data;
          if(l_it->border->client.icccm.class &&
             !strcmp(l_it->border->client.icccm.class,
                     it->border->client.icccm.class)) break;
          l_it = NULL;
        }
    }

  if(l_it)
    box->items = evas_list_prepend_relative(box->items, it, l_it);
  else
    box->items = evas_list_prepend(box->items, it);

  it->usable = 1;
  ngi_box_item_show(ng, it, 0);
  
  if (bd->desktop && bd->desktop->x && (val = ecore_hash_get(bd->desktop->x, "X-ItaskNG-SingleInstance")))
    it->is_single_instance_app = atoi(val);
  else
    it->is_single_instance_app = 0;

  edje_object_signal_emit(it->obj, "e,state,item_show", "e");

  //printf("_ngi_taskbar_item_new end\n");
  
}


Evas_Object *
_ngi_taskbar_border_icon_add(E_Border *bd, Evas *evas, int size)
{
  Evas_Object *o;
 
  E_OBJECT_CHECK_RETURN(bd, NULL);
  E_OBJECT_TYPE_CHECK_RETURN(bd, E_BORDER_TYPE, NULL);

  o = NULL;
  if (bd->internal)
    {
      o = edje_object_add(evas);
      if (!bd->internal_icon) 
	e_util_edje_icon_set(o, "enlightenment/e");
      else
	{
	  if (!bd->internal_icon_key)
	    {
	      char *ext;
	      ext = strrchr(bd->internal_icon, '.');
	      if ((ext) && ((!strcmp(ext, ".edj"))))
		{
		  if (!edje_object_file_set(o, bd->internal_icon, "icon"))
		    e_util_edje_icon_set(o, "enlightenment/e");	       
		}
	      else if (ext)
		{
		  evas_object_del(o);
		  o = e_icon_add(evas);
		  e_icon_file_set(o, bd->internal_icon);
		}
	      else 
		{
		  if (!e_util_edje_icon_set(o, bd->internal_icon))
		    e_util_edje_icon_set(o, "enlightenment/e"); 
		}
	    }
	  else
	    {
	      edje_object_file_set(o, bd->internal_icon,
				   bd->internal_icon_key);
	    }
	}
      return o;
    }
  if ((e_config->use_app_icon) && (bd->icon_preference != E_ICON_PREF_USER) && (bd->client.netwm.icons))
    {
      int i, diff, tmp, found = 0;
      o = e_icon_add(evas);

      diff = abs(bd->client.netwm.icons[0].width - size);

      for (i = 1; i < bd->client.netwm.num_icons; i++)
	{
	  if ((tmp = abs(bd->client.netwm.icons[i].width - size)) < diff)
	    {
	      diff = tmp;
	      found = i;
	    }
	}
	     
      e_icon_data_set(o, bd->client.netwm.icons[found].data,
		      bd->client.netwm.icons[found].width,
		      bd->client.netwm.icons[found].height);
      e_icon_alpha_set(o, 1);
      return o;
    }
  if (!o)
    {
      if ((bd->desktop) && (bd->icon_preference != E_ICON_PREF_NETWM))
	{
	  o = e_util_desktop_icon_add(bd->desktop, "256", evas);
	  if (o)
	    {
	      return o;
	    }
	     
	  o = e_util_desktop_icon_add(bd->desktop, "128", evas);
	  if (o)
	    {
	      return o;
	    }
	     
	  o = e_util_desktop_icon_add(bd->desktop, "48", evas);
	  if (o)
	    {
	      return o;
	    }
       
	  char size_str[256];
	  snprintf(size_str, 256, "%dx%d", size);
	     
	  o = e_util_desktop_icon_add(bd->desktop, size_str, evas);
	  if (o)
	    {
	      return o;
	    }
	}
  
      else if (bd->client.netwm.icons)
	{
	  int i, diff, tmp, found = 0;
	  o = e_icon_add(evas);

	  diff = abs(bd->client.netwm.icons[0].width - size);

	  for (i = 1; i < bd->client.netwm.num_icons; i++)
	    {
	      if ((tmp = abs(bd->client.netwm.icons[i].width - size)) < diff)
		{
		  diff = tmp;
		  found = i;
		}
	    }
	     
	  e_icon_data_set(o, bd->client.netwm.icons[found].data,
			  bd->client.netwm.icons[found].width,
			  bd->client.netwm.icons[found].height);
	  e_icon_alpha_set(o, 1);
	  return o;
	}
    }
   
  o = edje_object_add(evas);
  e_util_edje_icon_set(o, "enlightenment/unknown");
  return o;
}



static void
_ngi_taskbar_item_set_icon(Ngi_Item *it)
{
  //if (it->border->desktop && it->border->desktop->icon)
  // it->o_icon = e_util_icon_theme_icon_add(it->border->desktop->icon, "128x128", it->box->ng->win->evas);
  
  //if(!it->o_icon) 
  // it->o_icon = e_border_icon_add(it->border, it->box->ng->win->evas);
  
  it->o_icon = _ngi_taskbar_border_icon_add(it->border, it->box->ng->win->evas, 128);
  
  int w,h;
  
  evas_object_geometry_get(it->o_icon,NULL,NULL, &w, &h);
  
  //  evas_object_
  //printf("%dx%d\n",w,h);
  
  edje_object_part_swallow(it->obj, "e.swallow.content", it->o_icon);
  evas_object_pass_events_set(it->o_icon, 1);
  evas_object_show(it->o_icon);

  // if (it->border->desktop && it->border->desktop->icon)
  //  it->o_icon2 = e_util_icon_theme_icon_add(it->border->desktop->icon, "128x128", it->box->ng->win->evas);

  //if(!it->o_icon2) 
  //  it->o_icon2 = e_border_icon_add(it->border, it->box->ng->win->evas);
  
  it->o_icon2 = _ngi_taskbar_border_icon_add(it->border, it->box->ng->win->evas, 128);

  edje_object_part_swallow(it->over, "e.swallow.content", it->o_icon2);
  evas_object_pass_events_set(it->o_icon2, 1);

  if (it->border->iconic)
    edje_object_signal_emit(it->obj, "e,state,item_iconify", "e");
}

static void
_ngi_taskbar_item_set_label(Ngi_Item *it)
{
  const char *title;

  if(it->label)
    {
      evas_stringshare_del(it->label);
      it->label = NULL;
    }

  title = e_border_name_get(it->border);

  if ((!title) || (!title[0]))
    title  = N_("No name!");

  unsigned int max_len = 36;

  if (strlen(title) > max_len)
    {
      char *abbv;
      const char *left, *right;

      abbv = (char *) calloc(E_CLIENTLIST_MAX_CAPTION_LEN + 4, sizeof(char));
      left = title;
      right = title + (strlen(title) - (max_len / 2));

      strncpy(abbv, left, max_len / 2);
      strncat(abbv, "...", 3);
      strncat(abbv, right, max_len / 2);

      it->label =  evas_stringshare_add(abbv);
    }
  else
    {
      it->label = evas_stringshare_add(title);
    }
}


/* **************************  ITEM  CALLBACKS  ************************** */

static void
_ngi_taskbar_item_cb_free(Ngi_Item *it)
{
  it->box->items = evas_list_remove(it->box->items, it);

  ngi_item_del_icon(it);
  evas_object_del(it->obj);
  evas_object_del(it->over);

  if(it->label) evas_stringshare_del(it->label);

  if(it->overlay_signal_timer) ecore_timer_del(it->overlay_signal_timer);
  it->overlay_signal_timer = NULL;

  free(it);
  it = NULL;
}

static void
_ngi_taskbar_item_cb_mouse_down(Ngi_Item *it, Ecore_X_Event_Mouse_Button_Down *ev)
{
  Evas_Coord x, y, w, h;
  E_Border *bd;
  Ng * ng;
  
  if(!it->usable) return;

  ng = it->box->ng;
  bd = it->border;

  if (ev->button == 3 && bd)
    {
      int dir = E_MENU_POP_DIRECTION_AUTO;
      evas_object_geometry_get(it->obj, &x, &y, &w, &h);

      if(ng->cfg->stacking != on_desk)
        {
          x += ng->win->x + ng->zone->x;
          y += ng->win->y + ng->zone->y;
        }

      ngi_freeze(ng);
      ITEM_MOUSE_OUT(it);

      switch(ng->cfg->orient)
	{
	case E_GADCON_ORIENT_TOP:
	  dir =  E_MENU_POP_DIRECTION_DOWN;
          y += h;
          break;
	case E_GADCON_ORIENT_BOTTOM:
	  dir =  E_MENU_POP_DIRECTION_UP;
	  break;
	case E_GADCON_ORIENT_LEFT:
          dir =  E_MENU_POP_DIRECTION_RIGHT;
	  x +=w;
	  break;
	case E_GADCON_ORIENT_RIGHT:
	  dir =  E_MENU_POP_DIRECTION_LEFT;
	  break;
	}
      ngi_border_menu_show(it->box, it->border, x, y, dir, ev->time);
      e_util_evas_fake_mouse_up_later(ng->win->evas, ev->button);
    }

  /*  else if (ev->button == 1 && ev->flags & EVAS_BUTTON_DOUBLE_CLICK)
    {
      if(bd->fullscreen) 
        {
          it->border_was_fullscreen = 1;
          e_border_unfullscreen(bd);
        }

      e_border_iconify(bd);
    }*/
  else it->mouse_down = 1;
}

void
ngi_taskbar_item_border_show(Ngi_Item *it, int to_desk)
{
  E_Border *bd;
  E_Desk *desk;
  bd = it->border;

  if(to_desk)
    {    
      desk = e_desk_current_get(it->border->zone);

      if(desk != it->border->desk && !it->border->sticky)
        {
          e_desk_show(it->border->desk);
        }
    }
  
  if(bd->shaded)
    {
      e_border_unshade(it->border, E_DIRECTION_UP);
    }

  if(!bd->visible)
    {
      e_border_show(it->border);
    }

  if(bd->iconic)
    {
      e_border_uniconify(it->border);

      if(it->border_was_fullscreen) /* ditto */
        {
          e_border_fullscreen(bd, (E_Fullscreen) e_config->fullscreen_policy);
          it->border_was_fullscreen = 0;
        }
    }
  else
    {
      e_border_raise(bd);
    }

  e_border_focus_set(bd, 1, 1);
}

static void
_ngi_taskbar_item_cb_mouse_up(Ngi_Item *it, Ecore_X_Event_Mouse_Button_Up *ev)
{
  if(!it->mouse_down) return;
  if(!it->usable) return;

  it->mouse_down = 0;
 
  if (ev->button == 1 && ev->double_click && !it->border->iconic)
    {
      e_border_hide(it->border, 1);
    }
  else if (ev->button == 1)
    {
      ngi_taskbar_item_border_show(it, 1);
    }
}

static void
_ngi_taskbar_item_cb_drag_start(Ngi_Item *it)
{
  E_Drag *d;
  Evas_Object *o;
  Evas_Coord x, y, w, h, px, py;
  Ng *ng = it->box->ng;
  
  if(!it->usable) return;
  if(!it->border) return;

  edje_object_signal_emit(ng->label, "e,state,label_hide", "e");

  evas_object_geometry_get(it->o_icon, &x, &y, &w, &h);

  const char *drag_types[] = { "enlightenment/border" };

  d = e_drag_new(ng->zone->container, x, y, drag_types, 1,
                 it->border, -1, NULL, _ngi_taskbar_item_cb_drag_finished);

  if(ngi_config->use_composite)
    {
      ecore_evas_alpha_set(d->ecore_evas, 1);
      d->evas_win  = ecore_evas_software_x11_window_get(d->ecore_evas);
    }

  o = e_border_icon_add(it->border, e_drag_evas_get(d));

  e_object_ref(E_OBJECT(it->border));

  e_drag_object_set(d, o);

  e_drag_resize(d, w, h);

  if(ng->cfg->stacking == on_desk)
    ecore_x_pointer_xy_get(ng->zone->container->win, &px, &py);
  else
    ecore_x_pointer_xy_get(ng->win->evas_win, &px, &py);

  e_drag_start(d, px, py);

  e_util_evas_fake_mouse_up_later(ng->win->evas, 1); /*XXX?*/
}

static void
_ngi_taskbar_item_cb_drag_finished(E_Drag *drag, int dropped)
{
  E_Border *bd = (E_Border*) drag->data;
  
  if (!dropped)
    {

      /* TODO _ngi_taskbar_border_find
         ngi_taskbar_item_border_show()*/
      E_Desk *desk = e_desk_current_get(bd->zone);

      if(desk != bd->desk)
	e_border_desk_set(bd, desk);

      if(bd->iconic)
        {
          e_border_uniconify(bd);
        }
      else
        {
          e_border_raise(bd);
        }
      if(bd->shaded)
        {
          e_border_unshade(bd, E_DIRECTION_UP);
        }
    }

  e_object_unref(E_OBJECT(bd));
}
